<?php

namespace trapp\ExtJSAdminBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Response;

use trapp\ExtJSAdminBundle\Exception\FieldException;

class AdminController extends Controller
{
    public function indexAction()
    {
        $map = array();
        $items = array();
        $flat = array();
        $em = $this->get('doctrine.orm.entity_manager');
        $metaFactory = $em->getMetaDataFactory();
        $metaData = $metaFactory->getAllMetaData();
       
        foreach($metaData as $meta) {

            $namePart = explode('\\',$meta->name);
            $namePartCount = count($namePart);
            $current = '';
            
            for($i=0; $i<$namePartCount; $i++) {

                $parent = $current;
                if ($i>0) {
                    $current .= '\\';
                }
                $current .= $namePart[$i];
               
                if(!isset($items[$current])) {
                    $items[$current] = array(
                        'text' => $namePart[$i],
                        'id' => $current,
                        'ownAdmin' => false
                    );
                    if($parent != '') {
                        if(!isset($map[$parent]['children'])) {
                            $map[$parent]['children'] = array();
                        }
                        $map[$parent]['children'][] = &$items[$current];
                    }
                }
                if($i == $namePartCount-1) {
                    $items[$current]['leaf'] = true;
                    $felder = $meta->fieldMappings;
                    if (isset($meta->associationMappings)) {
                        foreach ($meta->associationMappings as $key=>$value) {
                            $meta->associationMappings[$key]['type'] = 'ref';
                        }
                        $felder = array_merge($felder, $meta->associationMappings);
                    }

                    $definition = array(
                        'fieldMappings' => $felder,
                        'identifier'    => $meta->identifier[0]
                    );

                    $items[$current]['data'] = $definition;
                    $flat[$current] = $definition;
                }
                if (!isset($map[$current])) {
                    $map[$current] = &$items[$current];
                }
            }
        }

        $keys = array_keys($map);
        $objects = json_encode(array($map[$keys[0]]));
        $flat = json_encode($flat);
        return $this->render('ExtJSAdminBundle:Admin:index.html.twig', array('objects' => $objects, 'flat' => $flat));
    }

    public function getDataAction($entity)
    {
        $em = $this->get('doctrine.orm.entity_manager');
        $request = $this->get('request');

        $metaFactory = $em->getMetaDataFactory();
        $meta = $metaFactory->getMetadataFor($entity);

        $start = $request->request->get('start') ? $request->request->get('start') : null;
        $limit = $request->request->get('limit') ? $request->request->get('limit') : null;
        $order = $request->request->get('sort')  ? $request->request->get('sort')  : $meta->identifier[0];
        $dir   = $request->request->get('dir')   ? $request->request->get('dir')   : 'ASC';
        $query = $request->request->get('query') ? $request->request->get('query') : null;

        

        $join = '';
        $joinCount = 1;
        $selectItems = '';

        if (isset($meta->associationMappings)) {
            foreach ($meta->associationMappings as $key=>$val) {
                $join .= ' JOIN u.' . $key . ' j' . $joinCount;
                $selectItems .= ', j' . $joinCount;
                $joinCount++;
            }
        }
        
        $q = $em->createQuery(
            sprintf(
                'SELECT u%s FROM %s u%s ORDER BY u.%s %s',
                $selectItems,
                $entity,
                $join,
                $order,
                $dir
            )
        );

        // ouch... This should be done in a more elegant way.
        $res = $q->getResult(\Doctrine\ORM\Query::HYDRATE_ARRAY);
        $total = count($res);
        
        $q->setFirstResult($start);
        $q->setMaxResults($limit);
        $dataset = $q->getResult(\Doctrine\ORM\Query::HYDRATE_ARRAY);

        return new Response(
            json_encode(
                array(
                    'total' => $total,
                    'objects' => $dataset
                )
            )
        );
    }

    public function saveAction()
    {
        $response = array();
        
        try {

            $request = $this->get('request');

            if (!$request->request->get('_submitObject')) {
                throw new \Exception('The entity name must be supplied.');
            }
            if ($request->request->get('_submitId') === null) {
                throw new \Exception('The entity id must be supplied.');
            }


            $em = $this->get('doctrine.orm.entity_manager');
            
            if ($request->request->get('_submitId') != 0) {
                $entity = $em->find($request->request->get('_submitObject'), $request->request->get('_submitId'));
                if (!$entity) {
                    throw new \Exception('Entity ' . $request->request->get('_submitObject') . ' with identifier ' . $request->request->get('_submitId') . ' doesn\'t exist.');
                }
            } else {
                $className = $request->request->get('_submitObject');
                if (!\class_exists($className)) {
                    throw new \Exception('Entity class ' . $className . ' doesn\'t exist.');
                }
                $entity = new $className();
                $em->persist($entity);
            }

            // Load metadata.
            $metaFactory = $em->getMetaDataFactory();
            $meta = $metaFactory->getMetadataFor($request->request->get('_submitObject'));

            foreach($request->request->All() as $key => $value) {

                if ($key == '_submitObject' || $key == '_submitId') {
                    continue;
                }

                if (isset($meta->associationMappings) && isset($meta->associationMappings[$key])) {
                    $target = $em->find($meta->associationMappings[$key]['targetEntity'], $value);
                    if (!$entity) {
                        throw new \Exception('Associated Entity ' . $meta->associationMappings[$key]['targetEntity'] . ' for field ' . $key . ' with identifier ' . $value . ' doesn\'t exist.');
                    }
                    $value = $target;
                }

                $setterName = 'set' . \ucfirst($key);
                if (\method_exists($entity, $setterName)) {
                    $entity->$setterName($value);
                } else {
                    
                }
            }

            $em->flush();

            $response['success'] = true;

        } catch (FieldException $e) {
            $response = array(
                'success' => false,
                'errors' => $e->getArray()
            );
        } catch (\Exception $e) {
            $response = array(
                'success' => false,
                'error' => $e->getMessage()
            );
        }
        return new Response(json_encode($response));

    }

    public function deleteAction()
    {
        $response = array();

        try {

            $request = $this->get('request');

            if (!$request->request->get('_submitObject')) {
                throw new \Exception('The entity name must be supplied.');
            }
            if (!$request->request->get('_submitId')) {
                throw new \Exception('The entity id must be supplied.');
            }
            if (!is_array($request->request->get('_submitId'))) {
                throw new \Exception('The entity id must be an array.');
            }

            $em = $this->get('doctrine.orm.entity_manager');

            foreach($request->request->get('_submitId') as $id) {
                $entity = $em->find($request->request->get('_submitObject'), $id);

                if (!$entity) {
                    throw new \Exception('Entity ' . $request->request->get('_submitObject') . ' with identifier ' . $id . ' doesn\'t exist.');
                }

                $em->remove($entity);

            }

            $em->flush();

            $response['success'] = true;

        } catch (\Exception $e) {
            $response = array(
                'success' => false,
                'error' => $e->getMessage()
            );
        }
        return new Response(json_encode($response));
    }
}
